-----------------------------------------------------------------------------------------------
-- Client Lua Script for KuronaBars
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- KuronaBars Module Definition
-----------------------------------------------------------------------------------------------
local KuronaBars = {} 
local Version = 0.13
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function KuronaBars:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function KuronaBars:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- KuronaBars OnLoad
-----------------------------------------------------------------------------------------------
function KuronaBars:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("KuronaBars.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("Sprites.xml","KBarSprite")

end

-----------------------------------------------------------------------------------------------
-- KuronaBars OnDocLoaded
-----------------------------------------------------------------------------------------------
function KuronaBars:OnDocLoaded()

	--Skilbar hide
	Apollo.SetConsoleVariable("hud.skillsBarDisplay",2)
	Apollo.SetConsoleVariable("hud.mountButtonDisplay",2)
	Apollo.SetConsoleVariable("hud.ResourceBarDisplay", 1)
	Apollo.SetConsoleVariable("hud.secondaryLeftBarDisplay",2)
	Apollo.SetConsoleVariable("hud.secondaryRightBarDisplay",2)

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.Bar1 = Apollo.LoadForm(self.xmlDoc, "KuronaBar1", "FixedHudStratum", self)
		self.Options = Apollo.LoadForm(self.xmlDoc, "KuronaBarsOptions", nil, self)
		self.EditModeForm = Apollo.LoadForm(self.xmlDoc, "EditModeForm", nil, self)
	    --self.Clicker = Apollo.LoadForm(self.xmlDoc, "ClickCommander", nil, self)
		
		self.wndBar2 = Apollo.LoadForm(self.xmlDoc, "Bar2ButtonContainer", "FixedHudStratum", self)
		self.wndBar3 = Apollo.LoadForm(self.xmlDoc, "Bar3ButtonContainer", "FixedHudStratum", self)
		--self.FloatingActionBar = Apollo.LoadForm(self.xmlDoc, "FloatingActionBar", "FixedHudStratum", self)

		self.VehicleBarFrame = self.Bar1:FindChild("VehicleBarFrame")
		self.FloatingActionBar = self.Bar1:FindChild("FloatingActionBar")
		self.Controls = self.EditModeForm:FindChild("ControlsWindow")
		if self.Bar1 == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		
 		Apollo.RegisterEventHandler("CharacterCreated", "OnReady", self)	
		Apollo.RegisterEventHandler("UnitEnteredCombat", 						"OnUnitEnteredCombat", self)
		Apollo.RegisterEventHandler("ChangeWorld", 							"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("ChatZoneChange", 							"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("PlayerChanged", 							"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("ResolutionChanged",						"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("ApplicationWindowSizeChanged", 			"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("OptionsUpdated_HUDPreferences", 			"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("PlayerLevelChange", 						"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("UpdateInventory", 							"OnUpdateInventory", self)
		Apollo.RegisterEventHandler("ShowActionBarShortcut", "ShowABWindow", self)
		Apollo.RegisterEventHandler("AbilityBookChange", 		"OnAbilityBookChange", self)
		Apollo.RegisterTimerHandler("KBarAB_CacheTimer",	"UpdateAbilityBook", self)
		Apollo.RegisterTimerHandler("PlayerTimer",	"OnReady", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("KuronaBarsClick", "OnKuronaBarsMenuClick", self)
		Apollo.RegisterEventHandler("GenericEvent_CollectablesReady", "OnReady", self)


		self.Options:Show(false,true)
		self.SnapCount = 0
		self.ControlType = 1
	
--	self.ControlType = self.Controls:GetRadioSel("PosSize")
	 self.Controls:SetRadioSel("PosSize", 1)
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		self.Options:FindChild("VersionWindow"):SetText("KuronaBars Beta: V"..Version)
		Apollo.RegisterSlashCommand("kbars", "OnKuronaBarsOn", self)

	if self.tSettings == nil then
		if self.tSettings == nil then
			self.tSettings = self:DefaultTable()
			self:OnKuronaBarsOn()
		end
	end
	
		if self.tSettings.OptionsPos then
			self.Options:SetAnchorOffsets(unpack(self.tSettings.OptionsPos))
		end
		if self.tSettings.EditModePos then
			self.EditModeForm:SetAnchorOffsets(unpack(self.tSettings.EditModePos))
		end

		-- Do additional Addon initialization here
	end
	self:FillOptions()
	self.CanScale = false
	self:RedrawActionBarShortcut()
	self:HideActionBar()
	if GameLib.GetPlayerUnit() ~= nil then
--		Apollo.CreateTimer("PlayerTimer", 1, false)
		self:OnReady()
	end
end

-----------------------------------------------------------------------------------------------
-- KuronaBars Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/kbars"
function KuronaBars:OnKuronaBarsOn(strArg)
	self.Options:Invoke() -- show the window
end


-----------------------------------------------------------------------------------------------
-- KuronaBarsForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function KuronaBars:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function KuronaBars:OnCancel()
	self.wndMain:Close() -- hide the window
end


---------------------------------------------------------------------------------------------------
-- ActionBarButtonForm Functions
---------------------------------------------------------------------------------------------------

function KuronaBars:OnGenerateTooltip(wndControl, wndHandler, eType, arg1, arg2)

	local xml = nil
	if eType == Tooltip.TooltipGenerateType_ItemInstance then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_ItemData then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_GameCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Macro then
		xml = XmlDoc.new()
		xml:AddLine(arg1)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Spell then

		if Tooltip ~= nil and Tooltip.GetSpellTooltipForm ~= nil then
			Tooltip.GetSpellTooltipForm(self, wndControl, arg1, {bTiers = true})
		end
	elseif eType == 4 then
		if wndControl == wndHandler then
		if wndControl:GetAbilityTierId() and splTarget:GetId() ~= wndControl:GetAbilityTierId() then
			splTarget = GameLib.GetSpell(wndControl:GetAbilityTierId())
		end
		Tooltip.GetSpellTooltipForm(self, wndHandler, splTarget, {bTiers = true})
		end
	elseif eType == Tooltip.TooltipGenerateType_PetCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	end
end

---------------------------------------------------------------------------------------------------
-- KuronaBar1 Functions
---------------------------------------------------------------------------------------------------

function KuronaBars:OnResize( wndHandler, wndControl )
	if wndControl:GetName() == "KuronaBar1" then
		local height = wndHandler:GetHeight()
		--self.Cont:SetScale(height / 110)		
	end
end

-----------------------------------------------------------------------------------------------
-- KuronaBars Instance
-----------------------------------------------------------------------------------------------
local KuronaBarsInst = KuronaBars:new()
KuronaBarsInst:Init()


function KuronaBars:InitializeBars()
	if self.EditMode then return end
	self.LastButton = nil
		for i=1,8 do
			if self.tSettings.SavedButtons["Button"..i] then
				self.tSettings.SavedButtons["Button"..i].Button = nil
			end
		end
--	self.tSettings.SavedButtons = nil
--	self.tSettings.SavedButtons = {}
	
	if self.tSettings.SavedButtons["MainBar"] and self.tSettings.SavedButtons["MainBar"].Pos then
		self.Bar1:SetAnchorOffsets(unpack(self.tSettings.SavedButtons["MainBar"].Pos))
	elseif self.tSettings.SavedButtons["MainBar"] then
		self.Bar1:SetAnchorOffsets(-345,-120,345,20)
	else
		self.Bar1:SetAnchorOffsets(-345,-120,345,20)
	end
	self.player = GameLib.GetPlayerUnit()
	if self.player == nil then Print("no play") return end
	self.tCurrentActionSet = ActionSetLib.GetCurrentActionSet()
	if self.tCurrentActionSet == nil then Print("nil set") return end
	self.Bar1:Show(true)
	local cont = self.Bar1:FindChild("ContentWindow")
	self.Cont = cont
	cont:DestroyChildren()
	-- secondarybars
	self.wndBar2:GetChildren()[1]:DestroyChildren()
	self.wndBar3:GetChildren()[1]:DestroyChildren()
	
	if self.tSettings.Bar2Pos then 
		self.wndBar2:SetAnchorOffsets(unpack(self.tSettings.Bar2Pos))
	end
	if self.tSettings.Bar3Pos then 
		self.wndBar3:SetAnchorOffsets(unpack(self.tSettings.Bar3Pos))
	end
	
	
	self.ButtonList = {}
	local bShow = not self.VehicleBarFrame:IsShown()

	
		--Recall
		local wndCurr = Apollo.LoadForm(self.xmlDoc, "InnateButtonForm", cont, self)
		wndCurr:Show(true,true)
		self.RecallButton = wndCurr
		local wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")		
		wndActionBarBtn:SetContentId(self.tSettings.nSelectedRecall or 18)
		
		self.wndRecallPopoutFrame = wndCurr:FindChild("FlyoutList")
		wndCurr:FindChild("ArrowUpButton"):AttachWindow(self.wndRecallPopoutFrame)

		
		if self.tSettings.SavedButtons["RecallButton"] == nil then
			self.wasReset = true
			self.tSettings.SavedButtons["RecallButton"] = {}
		end
		self.tSettings.SavedButtons["RecallButton"].Button = wndCurr
		--self.tSettings.SavedButtons["RecallButton"].Button:SetName("RecallButton")
		self.tSettings.SavedButtons["RecallButton"].Content = wndActionBarBtn
		self.tSettings.SavedButtons["RecallButton"].Spell = wndActionBarBtn:GetContent().spell
		self.tSettings.SavedButtons["RecallButton"].Name = "RecallButton"
		
		if self.tSettings.SavedButtons["RecallButton"].Pos == nil then  wndCurr:SetAnchorOffsets(0,0,50,50) end
	
			local cover = wndCurr:FindChild("Icon")
			
			if  wndActionBarBtn:GetContent().spell then
				cover:SetSprite(wndActionBarBtn:GetContent().spell:GetIcon())
				cover:SetText(wndActionBarBtn:GetContent().strText)
				wndActionBarBtn:Show(true)
			else
				cover:SetSprite("")			
				wndActionBarBtn:Show(false)
			end
		
		--Innate/stance
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "InnateButtonForm", cont, self)
			self.wndStancePopoutFrame = wndCurr:FindChild("FlyoutList")
			wndCurr:FindChild("ArrowUpButton"):AttachWindow(self.wndStancePopoutFrame)
			self.StanceButton = wndCurr
			if self.tSettings.SavedButtons["StanceButton"] == nil then
				self.tSettings.SavedButtons["StanceButton"] = {}
			end
			self.tSettings.SavedButtons["StanceButton"].Button = wndCurr
			self.tSettings.SavedButtons["StanceButton"].Spell = GameLib.GetCurrentClassInnateAbilitySpell()
			self.tSettings.SavedButtons["StanceButton"].Name = "StanceButton"
			--self.tSettings.SavedButtons["StanceButton"].Button:SetName("StanceButton")

			local wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
			wndActionBarBtn:SetContentId(2)
			
			local cover = wndCurr:FindChild("Icon")
			
			if  wndActionBarBtn:GetContent().spell then
				cover:SetSprite(wndActionBarBtn:GetContent().spell:GetIcon())
				cover:SetText(wndActionBarBtn:GetContent().strText)
				wndActionBarBtn:Show(true)
			else
				cover:SetSprite("")			
				wndActionBarBtn:Show(false)
			end
			
			if self.tSettings.SavedButtons["StanceButton"].Pos == nil then wndCurr:SetAnchorOffsets(0,0,50,50) end
			local strActionName = "CastInnateAbility"
			if Tooltip and Tooltip.GetSpellTooltipForm and spellObject then
				wndCurr:SetTooltipDoc(nil)
				Tooltip.GetSpellTooltipForm(self, wndCurr, spellObject)
			end
			wndCurr:Show(bShow,true)

			self.tSettings.SavedButtons["Button0"] = self.tSettings.SavedButtons["StanceButton"]
			self.tSettings.SavedButtons["Button0"].Button = wndCurr
	
	for idx = 1,34  do
	--Abilities
		if idx < 9 then
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "ActionBarButtonForm", cont, self)
			wndCurr:Show(bShow,true)
			if self.LastButton == nil then self.LastButton = wndCurr end
		
			local wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
			wndActionBarBtn:SetContentId(idx - 1)
			if self.tSettings.SavedButtons["Button"..idx] == nil then
				self.tSettings.SavedButtons["Button"..idx] = {}
				self.tSettings.SavedButtons["Button"..idx].Scale = 1
			end
			--if wndActionBarBtn:GetContent() then Print("x") end
			if wndActionBarBtn:GetContent() and  wndActionBarBtn:GetContent().spell then
				local strName = wndActionBarBtn:GetContent().spell:GetName()
				
				if wndActionBarBtn:GetContent().spell and self.tSettings.SavedButtons[strName] == nil then
					strName = wndActionBarBtn:GetContent().spell:GetName()
					self.tSettings.SavedButtons[strName] = {}
					self.tSettings.SavedButtons[strName].Pos = nil
				end
				if self.tSettings.SavedButtons[strName] == nil then
					self.tSettings.SavedButtons[strName] = {}
				end
			
				self.tSettings.SavedButtons[strName].Name = strName
					
				--self.tSettings.SavedButtons["Button"..idx] = {}
				self.tSettings.SavedButtons["Button"..idx].Button = wndCurr
	--			self.tSettings.SavedButtons["Button"..idx].Button:SetName("Button"..idx)
				
				self.tSettings.SavedButtons["Button"..idx].Content = wndActionBarBtn
				self.tSettings.SavedButtons["Button"..idx].Spell = wndActionBarBtn:GetContent().spell
				self.tSettings.SavedButtons["Button"..idx].Name = strName
						
		
			local cover = wndCurr:FindChild("Icon")
			
			if  wndActionBarBtn:GetContent().spell then
				cover:SetSprite(wndActionBarBtn:GetContent().spell:GetIcon())
				cover:SetText(wndActionBarBtn:GetContent().strText)
				wndActionBarBtn:Show(true)
			else
				cover:SetSprite("")			
				wndActionBarBtn:Show(false)
			end
			
		
			if Apollo.GetConsoleVariable("ui.actionBarTooltipsOnCursor") then
				wndActionBarBtn:SetTooltipType(Window.TPT_OnCursor)
			else
				wndActionBarBtn:SetTooltipType(Window.TPT_DynamicFloater)
			end
		else
			self.tSettings.SavedButtons["Button"..idx].Button = nil
			wndCurr:FindChild("Frame"):Show(false,true)
		end
		--Gadget
		elseif idx < 10 then
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "ActionBarButtonForm", cont, self)
			wndCurr:Show(bShow,true)
			--wndCurr:SetAnchorPoints(AnchorUnit,0,AnchorUnit+block,1)
			wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
			wndActionBarBtn:SetContentId(8)
			
			if self.tSettings.SavedButtons["GadgetButton"] == nil then
				self.tSettings.SavedButtons["GadgetButton"] = {}
			end
			self.tSettings.SavedButtons["GadgetButton"].Button = wndCurr
			self.tSettings.SavedButtons["GadgetButton"].Content = wndActionBarBtn
			self.tSettings.SavedButtons["GadgetButton"].Spell = GameLib.GetGadgetAbility()
			self.tSettings.SavedButtons["GadgetButton"].Name = "GadgetButton"
		--	self.tSettings.SavedButtons["GadgetButton"].Button:SetName("GadgetButton")			
			if wndActionBarBtn:GetContent().strIcon ~= "" then
			
				local cover = wndCurr:FindChild("Icon")
				if GameLib.GetPlayerUnit():GetEquippedItems()[11] then
					cover:SetSprite(GameLib.GetPlayerUnit():GetEquippedItems()[11]:GetIcon())
				end
				cover:SetText(wndActionBarBtn:GetContent().strText)
			else
				wndCurr:FindChild("Frame"):Show(false,true)
			end
			wndActionBarBtn:Show(true)
			self.tSettings.SavedButtons["Button9"] = self.tSettings.SavedButtons["GadgetButton"]
			self.tSettings.SavedButtons["Button9"].Button = wndCurr

--			wndCurr:FindChild("ButtonLabel"):SetText(GameLib.GetKeyBinding("CastGadgetAbility"))

			if self.tSettings.SavedButtons["GadgetButton"].Pos == nil then wndCurr:SetAnchorOffsets(0,0,50,50) end
		elseif idx < 11 then
		--Depreciated
		elseif idx < 23 then -- 11 to 22
			wndCurr = Apollo.LoadForm(self.xmlDoc, "ActionBarItemSmall", self.wndBar2:GetChildren()[1], self)
			wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
			wndActionBarBtn:SetContentId(idx + 1)

			--hide bars we can't draw due to screen size
			if (idx - 10) * wndCurr:GetWidth() > self.wndBar2:GetWidth() and self.wndBar2:GetWidth() > 0 then
				wndCurr:Show(false)
			end
		else -- 23 to 34
			wndCurr = Apollo.LoadForm(self.xmlDoc, "ActionBarItemSmall", self.wndBar3:GetChildren()[1], self)
			wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
			wndActionBarBtn:SetContentId(idx + 1)

			--hide bars we can't draw due to screen size
			if (idx - 22) * wndCurr:GetWidth() > self.wndBar3:GetWidth() and self.wndBar3:GetWidth() > 0 then
				wndCurr:Show(false)
			end
		end		
	end
	
		--Potions
		local wndCurr = Apollo.LoadForm(self.xmlDoc, "InnateButtonForm", cont, self)
		--local wndList = Apollo.LoadForm(self.xmlDoc, "FlyoutList", cont, self)
		self.wndPotionPopoutFrame = wndCurr:FindChild("FlyoutList")
		wndCurr:FindChild("ArrowUpButton"):AttachWindow(self.wndPotionPopoutFrame)
		self.PotionButton = wndCurr
		
		if self.tSettings.SavedButtons["PotionButton"] == nil then
			self.tSettings.SavedButtons["PotionButton"] = {}
		end
			self.tSettings.SavedButtons["PotionButton"].Button = wndCurr
		self.tSettings.SavedButtons["PotionButton"].Name = "PotionButton"
--		self.tSettings.SavedButtons["PotionButton"].Button:SetName("PotionButton")

		local wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
		wndActionBarBtn:SetContentId(27)
		self.tSettings.SavedButtons["PotionButton"].Spell = wndActionBarBtn:GetContent().spell
			
		
		if wndActionBarBtn:GetContent().spell then
			local cover = wndCurr:FindChild("Icon")
			cover:SetSprite(wndActionBarBtn:GetContent().spell:GetIcon())
			cover:SetText(wndActionBarBtn:GetContent().strText)
		end
		
		if self.tSettings.SavedButtons["PotionButton"].Pos == nil then wndCurr:SetAnchorOffsets(0,0,50,50) end
	
	--Path
		local wndCurr = Apollo.LoadForm(self.xmlDoc, "ActionBarButtonForm", cont, self)
	--	wndCurr:SetName("InnateButtonForm")
		wndCurr:Show(true,false)
		self.PathButton = wndCurr
		self.wndPathPopoutFrame = wndCurr:FindChild("FlyoutList")
	
		local wndActionBarBtn = wndCurr:FindChild("ActionBarBtn")
		wndActionBarBtn:SetContentId(9)
		wndCurr:FindChild("ArrowUpButton"):AttachWindow(self.wndPathPopoutFrame)
		wndCurr:FindChild("ArrowUpButton"):Show(true,true)

		if self.tSettings.SavedButtons["PathButton"] == nil then

		self.wasReset = true
			self.tSettings.SavedButtons["PathButton"] = {}
		end
		self.tSettings.SavedButtons["PathButton"].Button = wndCurr
		self.tSettings.SavedButtons["PathButton"].Content = wndActionBarBtn
		self.tSettings.SavedButtons["PathButton"].Spell = wndActionBarBtn:GetContent().spell
		self.tSettings.SavedButtons["PathButton"].Name = "PathButton"
--		self.tSettings.SavedButtons["PathButton"].Button:SetName("PathButton")			
		
		if self.tSettings.SavedButtons["PathButton"].Pos == nil then wndCurr:SetAnchorOffsets(0,0,50,50) end
	
		local cover = wndCurr:FindChild("Icon")
		if wndActionBarBtn:GetContent().strIcon ~= "" and wndActionBarBtn:GetContent().spell then
			cover:SetSprite(wndActionBarBtn:GetContent().spell:GetIcon())
			cover:SetText(wndActionBarBtn:GetContent().strText)
		else
			wndCurr:FindChild("Frame"):Show(false,true)
		end
		wndActionBarBtn:Show(true)
	
		
		self.Cont:ArrangeChildrenHorz(0)
		self.wndBar2:GetChildren()[1]:ArrangeChildrenHorz(0)
		self.wndBar3:GetChildren()[1]:ArrangeChildrenHorz(0)
		
		if self.wasReset then
			local l,t,r,b = self.tSettings.SavedButtons["PathButton"].Button:GetAnchorOffsets()
			self.tSettings.SavedButtons["PathButton"].Button:SetAnchorOffsets(l,t+5,r-5,b)
			local l,t,r,b = self.tSettings.SavedButtons["RecallButton"].Button:GetAnchorOffsets()
			self.tSettings.SavedButtons["RecallButton"].Button:SetAnchorOffsets(l+5,t+5,r,b)
		end
		
end



function KuronaBars:OnAbilityBookChange()
	self:HideActionBar()
	--Timer is needed because of multiple calls to the OnAbilityBookChange() callback
	Apollo.CreateTimer("KBarAB_CacheTimer", 0.1, false)
end

function KuronaBars:HideActionBar()
	Apollo.SetConsoleVariable("hud.skillsBarDisplay",2)
	Apollo.SetConsoleVariable("hud.mountButtonDisplay",2)
	Apollo.SetConsoleVariable("hud.ResourceBarDisplay", 2)
	Apollo.SetConsoleVariable("hud.secondaryLeftBarDisplay",2)
	Apollo.SetConsoleVariable("hud.secondaryRightBarDisplay",2)
end

	--GetSpellByName is a very costly call
function KuronaBars:UpdateAbilityBook()
	if GameLib.GetPlayerUnit() then	
		self:SaveButtonPositions()
		self:SetupBar()
		if self.tSettings.LASNums then
			self:SetButtonPositions()
		end
	end
	self:SaveButtonPositions()
	self:SetButtonPositions()
	--self.ActionBarFrame.wndMain:Show(false)
end


function KuronaBars:OnMouseOver( wndHandler, wndControl, x, y )
	--self.lastL,self.lastT = wndHandler:GetAnchorOffsets()
	if self.EditMode then
		if self.LastButton then
		self.LastButton:FindChild("Hilight"):Show(false)
--			self.LastButton:SetBGColor("ffffffff")
		end
		
		self.LastButton = wndHandler
--		self.LastButton:SetBGColor("ffff0080")
		self.LastButton:FindChild("Hilight"):Show(true)
		local wndActionBarBtn = self.LastButton:FindChild("ActionBarBtn")		
		if wndActionBarBtn:GetContent().spell then
			local strName = wndActionBarBtn:GetContent().spell:GetName()
			self.EditModeForm:FindChild("ButtonName"):SetText(strName)
		end
	
		self:UpdateEditFrame(wndHandler)
		self:ManualAdjust( wndHandler )
				
		return
	end
	
	if self.tSettings.AllowClick then return end

	if wndControl:GetName() == "ActionBarButtonForm" or wndControl:GetName() == "InnateButtonForm" then
	
		wndControl:FindChild("Icon"):Show(true)
		wndControl:FindChild("ActionBarBtn"):Show(false)
	end	
end


function KuronaBars:UpdateEditFrame(wndHandler)

end


function KuronaBars:OnMouseExit( wndHandler, wndControl, x, y )
if self.tSettings.AllowClick or self.EditMode then return end
	if wndControl:GetName() == "ActionBarButtonForm" or wndControl:GetName() == "InnateButtonForm" then
		wndControl:FindChild("Icon"):Show(false)
		wndControl:FindChild("ActionBarBtn"):Show(true)
	end
end

function KuronaBars:OnMouseOverBtn( wndHandler, wndControl, x, y )
if not wndControl or  (self.tSettings and self.tSettings.AllowClick) then return end
	if self.EditMode then return end
	if wndControl:GetName() == "ActionBarBtn" then
		wndControl:GetParent():FindChild("Icon"):Show(true)
		wndControl:GetParent():FindChild("ActionBarBtn"):Show(false)
	end
end


function KuronaBars:OnFrame()
	if self.player then
		self.bCombatState = self.player:IsInCombat()
	end
	if not self.EditMode and not self.VehicleBarShown then
	for i=1,8 do
		if self.tSettings.SavedButtons["Button"..i] then
			self:CheckCD("Button"..i)
		end
	end
	self:CheckCD("PotionButton")
	self:CheckCD("StanceButton")
	self:CheckCD("GadgetButton")
	end
	---self.ActionBarFrame.wndMain:Show(false)
end

function KuronaBars:CheckCD(button)
	if self.tSettings.SavedButtons[button] == nil then return end	
		--local button = self.tSettings[button].Button
		
		if self.tSettings.SavedButtons[button] and self.tSettings.SavedButtons[button].Button and self.tSettings.SavedButtons[button].Spell ~= nil and ((self.tSettings.CombatOnly and self.bCombatState) or (not self.tSettings.CombatOnly)) then
			local charges = self.tSettings.SavedButtons[button].Spell:GetAbilityCharges()
			local CD =  self.tSettings.SavedButtons[button].Spell:GetCooldownRemaining()
			local meter = self.tSettings.SavedButtons[button].Button:FindChild("ProgressBar")
						
			if CD ~= 0 then
				local CDT =  self.tSettings.SavedButtons[button].Spell:GetCooldownTime()
				if meter and not charges then
					meter:SetMax(CDT)
					meter:SetProgress(CD)
					meter:Show(true)
				elseif meter then
					meter:Show(false)			
				end
			elseif CD == 0 then
				if meter then
					meter:Show(false)
				end
			end
			
			local button = self.tSettings.SavedButtons[button].Button
			
			if self.tSettings.HideCoolDown then
				if  charges and charges.nChargesMax then
					if  charges.nChargesRemaining < 1 then
						CD = charges.fRechargePercentRemaining * charges.fRechargeTime
					else
						CD = 0
					end
				end			

				if CD < 3 and self.tSettings.HideCoolDown then			
					if not button:IsShown() then
						button:Show(true)					
					end
					if meter then
						meter:SetProgress(0)
					end
				elseif self.tSettings.HideCoolDown then
					if button:IsShown() then
						button:Show(false,false)
					end					
				end
			elseif not button:IsShown() then
				button:Show(true,false)
			end			
		else
			local button = self.tSettings.SavedButtons[button].Button
			if (self.tSettings.CombatOnly and self.bCombatState) and button and not button:IsShown() then
				button:Show(true,true)
			elseif (self.tSettings.CombatOnly and not self.bCombatState) and button and button:IsShown() then 
				button:Show(false,false)
			end
		end
	end


function KuronaBars:GetBaseWindow(item)
	if item:GetName() == "InnateButtonForm" then
		return item
		
	elseif item:GetParent() then
		self:GetBaseWindow(item:GetParent())
	else
		return item
	end
end

function KuronaBars:OnClickThru( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	if self.EditMode then return end
	if eMouseButton == 1 or eMouseButton == 0 then
		wndControl:GetParent():FindChild("Icon"):Show(false)
		wndControl:GetParent():FindChild("ActionBarBtn"):Show(true)
		if wndControl:GetParent():GetParent():GetName() == "InnateButtonForm" then
			wndControl:GetParent():GetParent():FindChild("FlyoutList"):Show(true,false)
		end
	end
end



function KuronaBars:OnClickUp( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	if self.EditMode then return end
	if wndControl:GetName() == "InnateButtonForm" or wndControl:GetName() == "ActionBarButtonForm"  then
	if eMouseButton == 1 or eMouseButton == 0 then
		wndControl:GetParent():FindChild("Icon"):Show(true)
		wndControl:GetParent():FindChild("ActionBarBtn"):Show(false)
		wndControl:SetStyle("IgnoreMouse",true)
	end
	end
end

function KuronaBars:SetButtonPositions()
--if not thisthis then return end
	self.ButtonList = {
	[1] = "Button1",
	[2] = "Button2",
	[3] = "Button3",
	[4] = "Button4",
	[5] = "Button5",
	[6] = "Button6",
	[7] = "Button7",
	[8] = "Button8",
	[9] = "StanceButton",
	[10] = "PotionButton",
	[11] = "GadgetButton",
	[12] = "PathButton",
	[13] = "RecallButton"
	}
	
	for k,idx in pairs(self.ButtonList) do
		if self.tSettings.SavedButtons[idx] and self.tSettings.SavedButtons[idx].Button and self.tSettings.SavedButtons[idx].Pos then
			if self.tSettings.LASNums then
				self.tSettings.SavedButtons[idx].Button:SetAnchorOffsets(unpack(self.tSettings.SavedButtons[idx].Pos))
			elseif self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos then
				self.tSettings.SavedButtons[idx].Button:SetAnchorOffsets(unpack(self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos))
			end			
			self.tSettings.SavedButtons[idx].Button:SetScale(self.tSettings.SavedButtons[idx].Scale)
			self.tSettings.SavedButtons[idx].Button:SetOpacity(0.75,23)
		end
	end

end

function KuronaBars:SaveButtonPositions()
	self.ButtonList = {
	[1] = "Button1",
	[2] = "Button2",
	[3] = "Button3",
	[4] = "Button4",
	[5] = "Button5",
	[6] = "Button6",
	[7] = "Button7",
	[8] = "Button8",
	[9] = "StanceButton",
	[10] = "PotionButton",
	[11] = "GadgetButton",
	[12] = "PathButton",
	[13] = "RecallButton"
	}
--[[	
	for k,idx in pairs(self.ButtonList) do
		if self.tSettings.SavedButtons[idx] then
			self.tSettings.SavedButtons[idx].Pos = nil
			self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos = nil
		end
	end
]]	
	for k,idx in pairs(self.ButtonList) do
		if self.tSettings.SavedButtons[idx] and self.tSettings.SavedButtons[idx].Name and self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name] and self.tSettings.SavedButtons[idx].Button ~= nil then
			local button = self.tSettings.SavedButtons[idx].Button
			self.tSettings.SavedButtons[idx].Pos = {button:GetAnchorOffsets()}
			self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos = {button:GetAnchorOffsets()}
			self.tSettings.SavedButtons[idx].Scale = button:GetScale()
		end
	end
	
	self.tSettings.SavedButtons["MainBar"] = {}
	self.tSettings.SavedButtons["MainBar"].Name = "MainBar"
	self.tSettings.SavedButtons["MainBar"].Pos = {self.Bar1:GetAnchorOffsets()}	
	self.tSettings.SavedButtons["MainBar"].Scale = 1	
	self.tSettings.OptionsPos = {self.Options:GetAnchorOffsets()}	
	self.tSettings.EditModePos = {self.EditModeForm:GetAnchorOffsets()}	
	self.tSettings.FloatBarPos = {self.FloatingActionBar:GetAnchorOffsets()}	
	self.tSettings.Bar2Pos = {self.wndBar2:GetAnchorOffsets()}	
	self.tSettings.Bar3Pos = {self.wndBar3:GetAnchorOffsets()}	
	
end

function KuronaBars:OnSave(eLevel,backup)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	self:SaveButtonPositions()

	
	self.ButtonList = {
	[1] = "Button1",
	[2] = "Button2",
	[3] = "Button3",
	[4] = "Button4",
	[5] = "Button5",
	[6] = "Button6",
	[7] = "Button7",
	[8] = "Button8",
	[9] = "StanceButton",
	[10] = "PotionButton",
	[11] = "GadgetButton",
	[12] = "PathButton",
	[13] = "RecallButton"
	}
	
	for k,idx in pairs(self.ButtonList) do
		if self.tSettings.SavedButtons[idx] then
			self.tSettings.SavedButtons[idx].Pos = nil
--			self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos = nil
		end
	end
	for k,idx in pairs(self.ButtonList) do
		if self.tSettings.SavedButtons[idx] and self.tSettings.SavedButtons[idx].Name and self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name] and self.tSettings.SavedButtons[idx].Button ~= nil then
		local button = self.tSettings.SavedButtons[idx].Button
			--Print(idx .." ".. button:GetName())
			self.tSettings.SavedButtons[idx].Pos = {button:GetAnchorOffsets()}
			--self.tSettings.SavedButtons[self.tSettings.SavedButtons[idx].Name].Pos = {button:GetAnchorOffsets()}
			self.tSettings.SavedButtons[idx].Scale = button:GetScale()
		end
	end

	
	local tsave = {}
	
	
	
	
	tsave = self.tSettings	
	
	
	Apollo.SetConsoleVariable("hud.skillsBarDisplay",1)
	Apollo.SetConsoleVariable("hud.mountButtonDisplay",1)
	Apollo.SetConsoleVariable("hud.secondaryLeftBarDisplay",1)
	Apollo.SetConsoleVariable("hud.secondaryRightBarDisplay",1)

	return tsave
end

function KuronaBars:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	self.tSettings = {}			
	local defaultsettings = self:DefaultTable()
		
	for k,v in pairs(saveData) do
	   	self.tSettings[k]=v
	end

	for k,v in pairs(defaultsettings) do
	   	if self.tSettings[k] == nil then
			 self.tSettings[k] = v
		end
	end
end

function KuronaBars:DefaultTable()
	local default = {
		CombatOnly = false,
		HideCoolDown = false,
		AllowClick = false,
		SavedButtons = {},
		LASNums = true
	}
	return default
end


function KuronaBars:OnSettingsChange( wndHandler, wndControl, eMouseButton )
	self.tSettings[wndHandler:GetName()] = wndHandler:IsChecked()
	self:SetupBar()
end

function KuronaBars:FillOptions()
--Checkboxes
	local cb = self.Options:FindChild("Checkboxes")
	local cbc = cb:GetChildren()

	for i = 1, #cbc do
		cbc[i]:SetCheck(self.tSettings[cbc[i]:GetName()])
	end
end

function KuronaBars:StartEditMode( wndHandler, wndControl, eMouseButton )

	for idx, bar in pairs (self.tActionBarsHorz) do
		if self.tActionBarsHorz[idx]:IsShown() then
			self.LastVisibleFloater = idx
			self.LastVisibleFloaterShort = self.tActionBarSettings[idx].nShortcuts
			--self.tActionBarsHorz[idx]:Show(false)
		end
	end

	self.EditMode = true
	self.Options:Close()
	self.EditModeForm:Invoke()
		
	for i=1,#self.ButtonList do
		self:EditButton(self.ButtonList[i],true)
	end
	self.Bar1:SetText("Limited Action Set Bar")	
	self.Bar1:SetStyle("IgnoreMouse",false)
	self.Bar1:SetStyle("Picture",true)
	self.Bar1:SetStyle("Moveable",true)
	self.Bar1:SetTooltip("Move All Ability Buttons")
	
	self.wndBar2:SetText("Left Secondary Action Bar")
	self.wndBar2:SetStyle("IgnoreMouse",false)
	self.wndBar2:SetStyle("Picture",true)
	self.wndBar2:SetStyle("Moveable",true)	
	self.wndBar2:SetTooltip("Movable")
	
	self.wndBar3:SetText("Right Secondary Action Bar")
	self.wndBar3:SetStyle("IgnoreMouse",false)
	self.wndBar3:SetStyle("Picture",true)
	self.wndBar3:SetStyle("Moveable",true)
	self.wndBar3:SetTooltip("Movable")
	
	self.FloatingActionBar:SetText("Floating Action Bar")
	self.FloatingActionBar:SetStyle("IgnoreMouse",false)
	self.FloatingActionBar:SetStyle("Picture",true)
	self.FloatingActionBar:SetStyle("Moveable",true)
	self.FloatingActionBar:SetStyle("Sizable",true)
	self.FloatingActionBar:SetTooltip("Floating ActionBar: Movable. This autocenters horizontally")
	self.FloatingActionBar:Show(true)
	self:ShowABWindow(7,true,7)
	--self.FloatingActionBar:GetChildren()[4]:Show(true)
	
	self:OnMouseOver( self.LastButton, self.LastButton, x, y )	
	
	
	self.Kpets = Apollo.GetAddon("KuronaPets")
	self.Kmounts = Apollo.GetAddon("KuronaMounts")
	
	if self.Kpets then
		self.Kpets:OnConsole()
		self.Kpets.wndMain:Show(false,true)
	end
	if self.Kmounts then
		self.Kmounts:OnConsole()
		self.Kmounts.wndMain:Show(false,true)
	end

--	self.EditModeForm:FindChild("WidthBox"):SetText(self.LastButton:GetWidth())
	
end

function KuronaBars:ExitEditMode( wndHandler, wndControl, eMouseButton )
	self.EditMode = false
	self.EditModeForm:Close()
	self.Options:Invoke()
	
	for i=1,#self.ButtonList do
		self:EditButton(self.ButtonList[i],false)
	end
	self.Bar1:SetText("")	
	self.Bar1:SetStyle("Moveable",false)
	self.Bar1:SetStyle("IgnoreMouse",true)
	self.Bar1:SetStyle("Picture",false)
	self.Bar1:SetTooltip("")
	self.wndBar2:SetText("")
	self.wndBar2:SetStyle("Moveable",false)
	self.wndBar2:SetStyle("IgnoreMouse",true)
	self.wndBar2:SetStyle("Picture",false)
	self.wndBar2:SetTooltip("")
	
	self.wndBar3:SetText("")
	self.wndBar3:SetStyle("Moveable",false)
	self.wndBar3:SetStyle("IgnoreMouse",true)
	self.wndBar3:SetStyle("Picture",false)
	self.wndBar3:SetTooltip("")
	
	self.FloatingActionBar:SetText("")		
	self.FloatingActionBar:SetStyle("IgnoreMouse",true)
	self.FloatingActionBar:SetStyle("Picture",false)
	self.FloatingActionBar:SetStyle("Moveable",false)
	self.FloatingActionBar:SetStyle("Sizable",false)
	self.FloatingActionBar:SetTooltip("")
	--self.FloatingActionBar:GetChildren()[4]:Show(false)
	self:ShowABWindow(7,false,7)

	
	if self.LastVisibleFloater then
--		self.LastVisibleFloater:Show(true,true)
		self.FloatingActionBar:Show(true,true)
--		self.tActionBarsHorz[self.LastVisibleFloater]:Show(true,true)
		self:ShowABWindow(self.LastVisibleFloater,true,self.LastVisibleFloaterShort)		
		self.LastVisibleFloater = nil
	else
		self.FloatingActionBar:Show(false,false)
	end

	
	if self.LastButton then self.LastButton:FindChild("Frame"):SetBGColor("ffffffff") end
	
	self.Kpets = Apollo.GetAddon("KuronaPets")
	self.Kmounts = Apollo.GetAddon("KuronaMounts")
	
	if self.Kpets then
		self.Kpets:OnOK()
	end
	if self.Kmounts then
		self.Kmounts:OnOK()
	end

	self.LastButton:FindChild("Hilight"):Show(false)
		
	
	self:SaveButtonPositions()
end


function KuronaBars:EditButton(button,state)
	local button = self.tSettings.SavedButtons[button].Button
	if button == nil then return end
	if state then
		button:SetStyle("Moveable",true)
		button:SetStyle("Sizable",true)
		--button:SetStyle("Picture",true)
		button:Show(true)
		button:FindChild("ActionBarBtn"):Show(false)
		button:FindChild("Icon"):Show(true)
		button:SetStyle("IgnoreMouse",false)
	else
		button:SetStyle("Moveable",false)
		button:SetStyle("Sizable",false)
		button:SetStyle("Picture",false)
		button:FindChild("ActionBarBtn"):Show(true)
		button:FindChild("Icon"):Show(false)
		button:SetStyle("IgnoreMouse",true)
		button:SetText("")
	end
	if not self.EditMode then
		self:OnMouseExit( button, button, 0, 0 )		
	end
end


function KuronaBars:ResetPositions( wndHandler, wndControl, eMouseButton )
--[[
	for i = 1, 9 do
		self.tSettings.SavedButtons["Button"..i] = nil
	end
	self.tSettings.SavedButtons["StanceButton"] = nil
	self.tSettings.SavedButtons["PotionButton"] = nil
	self.tSettings.SavedButtons["GadgetButton"] = nil
	self.tSettings.SavedButtons["RecallButton"] = nil
	self.tSettings.SavedButtons["PathButton"] = nil
	self.tSettings.SavedButtons["MainBar"] = nil
	]]
	for idx, button in pairs(self.tSettings.SavedButtons) do
		self.tSettings.SavedButtons[idx] = nil
	end
	self.tSettings.SavedButtons = nil
	self.tSettings.SavedButtons = {}	
	self.tSettings.SavedButtons = {}
	self.tSettings.FloatBarPos = {-200,-98,200,-48}
	self.tSettings.Bar2Pos = {77,-115,628,-20}
	self.tSettings.Bar3Pos = {-622,-115,-77,-20}
	
	self:SetupBar()
	
	self.Kpets = Apollo.GetAddon("KuronaPets")
	self.Kmounts = Apollo.GetAddon("KuronaMounts")
	
	if self.Kpets then
		self.Kpets:ResetPos()
	end
	if self.Kmounts then
		self.Kmounts:ResetPos()
	end

	
	
	--self:InitializeBars()
	self:SaveButtonPositions()
	self:FillOptions()	
end

function KuronaBars:OnButtonMove( wndHandler, wndControl, l, t, r, b )
	if self.ButtonMove then return end
	self.ButtonMove = true
	--[[
	--snapping
	local height = wndHandler:GetHeight()
	local width = wndHandler:GetWidth()
	--l = self:round(l,50)
	--t = self:round(t,50)
	--r = self:round(r,50)
	--b = self:round(b,50)
	local dir = "mid"
	local xdir = "mid"
	
	if self.lastL and self.lastL < l then
	dir = "left"
	elseif self.lastL and self.lastL > l then
	dir = "right"
	end
	
	if self.lastT and self.lastT < t then
		xdir = "up"
	elseif self.lastT and self.lastT > t then
		xdir = "down"
	end
	
		
	local fL = math.floor(l / 10)*10
	local cL = math.ceil(l / 10)*10
	
	local fT = math.floor(t / 10)*10
	local cT = math.ceil(t / 10)*10
	local snapX = l
	local snapY = t
	
	if Apollo.IsShiftKeyDown() then
		if dir == "left" then
			snapX = cL
		else
			snapX = fL
		end
		
		if xdir == "up" then
			snapY = cT
		else
			snapY = fT
		end
		self.SnapCount = 0
		wndHandler:SetAnchorOffsets(snapX,snapY,snapX+width,snapY+height)
	
	
else	
	--wndHandler:SetAnchorOffsets(self.lastL,self.lastT,self.lastsnapX+width,snapY+height)
end
	self.ButtonMove = false
	self.lastL = l
	self.lastT = t
	
		self:ManualAdjust( wndHandler, wndControl, eMouseButton )

	
	]]
end


function KuronaBars:OnButtonResize( wndHandler, wndControl )
	if self.EditMode and self.CanScale then
		self.CanScale = false
		local pos = {wndHandler:GetAnchorOffsets()}
		local width = wndHandler:GetWidth()
		if width < 10 then
			wndHandler:SetAnchorOffsets(pos[1],pos[2],pos[1]+10,pos[2]+10)
		end
		self:ManualAdjust( wndHandler, wndControl, eMouseButton )
		self.CanScale = true
		
	--[[
		self.CanScale = false
		local pos = {wndHandler:GetAnchorOffsets()}
		local scale = wndHandler:GetScale()
		--convert offsets to scale
		--local height = wndHandler:GetHeight() * scale
		local width = wndHandler:GetWidth()
--		height = height /  scale
		local newScale = (width / 50)
		--if newScale < 0.33 then newScale = scale end
		
		if (self.LastWidth ~= width) then
			self.CanScale = false
			self.LastWidth = width
			if width < (50) and scale <= 1 then
				newScale = 1
				wndHandler:SetAnchorOffsets(pos[1],pos[2],pos[3],pos[4])
			else			
				wndHandler:SetAnchorOffsets(pos[1],pos[2],pos[1]+50,pos[2]+50)
			end
			Print(newScale.." "..pos[2] * newScale)
			
			if newScale ~= scale then
				wndHandler:SetScale(newScale)
			end			
			
			--newScale =  math.ceil(newScale*100)*0.01
			--wndHandler:SetText(newScale.."X")
			self:ManualAdjust( wndHandler, wndControl, eMouseButton )
		end
		self.CanScale = true
		self.LastScale = nil
		]]
	end
end

function KuronaBars:OnReady()
	if GameLib:GetPlayerUnit() then
		self:SetupBar()
		Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
		Apollo.CreateTimer("KBarAB_CacheTimer", 2, false)
		self:SetupBar()
		self:HideActionBar()
	else
		Apollo.CreateTimer("PlayerTimer", 2, false)
	end
end

function KuronaBars:OnCharacterCreated()
	Apollo.CreateTimer("PlayerTimer", 2, false)
--	self:OnReady()
--	self:SetupBar()
end

function KuronaBars:SetupBar()
	self.player = GameLib.GetPlayerUnit()
	if self.player == nil then return end
	self.CanScale = false
	self:InitializeBars()	
	
	self:SetButtonPositions()
		
	self:RedrawPotions()
	self:RedrawStances()
	self:RedrawPaths()
	self:RedrawRecall()
	self:SaveButtonPositions()

	self.CanScale = true
end

function KuronaBars:RedrawPotions()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	local wndPotionPopout = self.wndPotionPopoutFrame --:FindChild("PotionPopoutList")
	wndPotionPopout:DestroyChildren()

	local tItemList = unitPlayer and unitPlayer:IsValid() and unitPlayer:GetInventoryItems() or {}
	local tSelectedPotion = nil
	local tFirstPotion = nil
	local tPotions = { }
	
	for idx, tItemData in pairs(tItemList) do
		if tItemData and tItemData.itemInBag and tItemData.itemInBag:GetItemCategory() == 48 then--and tItemData.itemInBag:GetConsumable() == "Consumable" then
			local itemPotion = tItemData.itemInBag

			if tFirstPotion == nil then
				tFirstPotion = itemPotion
			end

			if itemPotion:GetItemId() == self.tSettings.nSelectedPotion then
				tSelectedPotion = itemPotion
			end
			
			local idItem = itemPotion:GetItemId()

			if tPotions[idItem] == nil then
				tPotions[idItem] = 
				{
					itemObject = itemPotion,
					nCount = itemPotion:GetStackCount(),
				}
			else
				tPotions[idItem].nCount = tPotions[idItem].nCount + itemPotion:GetStackCount()
			end
		end
	end

	for idx, tData  in pairs(tPotions) do
		local wndCurr = Apollo.LoadForm(self.xmlDoc, "PotionBtn", wndPotionPopout, self)
	if (tData.nCount > 1) then wndCurr:FindChild("PotionBtnStackCount"):SetText(tData.nCount) end
		wndCurr:SetData(tData.itemObject)
		wndCurr:SetTooltipDoc(nil)
		local cover = wndCurr:FindChild("PotionBtnIcon")
		cover:SetSprite(tData.itemObject:GetIcon())
		
		Tooltip.GetItemTooltipForm(self, wndCurr, tData.itemObject, {})
	end

	if tSelectedPotion == nil and tFirstPotion ~= nil then
		tSelectedPotion = tFirstPotion
	end

	if tSelectedPotion ~= nil then
		GameLib.SetShortcutPotion(tSelectedPotion:GetItemId())
		self.PotionButton:FindChild("Icon"):SetSprite(tSelectedPotion:GetIcon())
		local testT = tSelectedPotion:GetDetailedInfo(Item.CodeEnumItemDetailedTooltip.Spell)
		self.tSettings.SavedButtons["PotionButton"].Spell = testT.tPrimary.arSpells[1].splData	
	end

	local nCount = #wndPotionPopout:GetChildren()
	if nCount > 0 then
		local nMax = 7
		local nMaxHeight = (wndPotionPopout:ArrangeChildrenVert(0) / nCount) * nMax
		local nHeight = wndPotionPopout:ArrangeChildrenVert(0)

		nHeight = nHeight <= nMaxHeight and nHeight or nMaxHeight

		local nLeft, nTop, nRight, nBottom = self.wndPotionPopoutFrame:GetAnchorOffsets()
		local scroll = 4
		if nCount > 7 then scroll = 20 end
		self.wndPotionPopoutFrame:SetAnchorOffsets(nLeft, nBottom - nHeight, nRight+scroll, nBottom)
	end

	--self.wndPotionPopoutFrame:Show(nCount > 0)
	self.PotionButton:FindChild("ArrowUpButton"):Show(#wndPotionPopout:GetChildren() > 1)	
end


function KuronaBars:OnUpdateInventory()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	if self.nPotionCount == nil then
		self.nPotionCount = 0
	end

	local nLastPotionCount = self.nPotionCount
	local tItemList = unitPlayer and unitPlayer:IsValid() and unitPlayer:GetInventoryItems() or {}
	local tPotions = { }

	for idx, tItemData in pairs(tItemList) do
		if tItemData and tItemData.itemInBag and tItemData.itemInBag:GetItemCategory() == 48 then--and tItemData.itemInBag:GetConsumable() == "Consumable" then
			local tItem = tItemData.itemInBag

			if tPotions[tItem:GetItemId()] == nil then
				tPotions[tItem:GetItemId()] = {}
				tPotions[tItem:GetItemId()].nCount=tItem:GetStackCount()
			else
				tPotions[tItem:GetItemId()].nCount = tPotions[tItem:GetItemId()].nCount + tItem:GetStackCount()
			end
		end
	end

	self.nPotionCount = 0
	for idx, tItemData in pairs(tPotions) do
		self.nPotionCount = self.nPotionCount + 1
	end

	if self.nPotionCount ~= nLastPotionCount then
		self:RedrawPotions()
	end
end

function KuronaBars:OnPotionBtn(wndHandler, wndControl)
	self.tSettings.nSelectedPotion = wndControl:GetData():GetItemId()

	self.wndPotionPopoutFrame:Show(false,false)
	self:RedrawPotions()
end


function KuronaBars:RedrawStances()
	local wndStancePopout = self.wndStancePopoutFrame
	wndStancePopout:DestroyChildren()

	local nCountSkippingTwo = 0
	for idx, spellObject in pairs(GameLib.GetClassInnateAbilitySpells().tSpells) do
		if idx % 2 == 1 then
			nCountSkippingTwo = nCountSkippingTwo + 1
			local strKeyBinding = GameLib.GetKeyBinding("SetStance"..nCountSkippingTwo) -- hardcoded formatting
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "StanceBtn", wndStancePopout, self)
			wndCurr:FindChild("StanceBtnIcon"):SetSprite(spellObject:GetIcon())
			
			wndCurr:SetData(nCountSkippingTwo)

			if Tooltip and Tooltip.GetSpellTooltipForm and spellObject then
				wndCurr:SetTooltipDoc(nil)
				Tooltip.GetSpellTooltipForm(self, wndCurr, spellObject)
			end
		end
	end

	local nHeight = wndStancePopout:ArrangeChildrenVert(0)
	local nLeft, nTop, nRight, nBottom = self.wndStancePopoutFrame:GetAnchorOffsets()
	self.wndStancePopoutFrame:SetAnchorOffsets(nLeft, nBottom - nHeight, nRight, nBottom)
	self.StanceButton:FindChild("ArrowUpButton"):Show(#wndStancePopout:GetChildren() > 1)
	self.StanceButton:FindChild("Icon"):SetSprite(GameLib.GetCurrentClassInnateAbilitySpell():GetIcon())
	self.tSettings.SavedButtons["StanceButton"].Spell = GameLib.GetCurrentClassInnateAbilitySpell()
end

function KuronaBars:OnStanceBtn(wndHandler, wndControl)
	self.wndStancePopoutFrame:Show(false)
	GameLib.SetCurrentClassInnateAbilityIndex(wndHandler:GetData())
	self.StanceButton:FindChild("Icon"):SetSprite(GameLib.GetCurrentClassInnateAbilitySpell():GetIcon())
end

function KuronaBars:OptionsClose( wndHandler, wndControl, eMouseButton )
self.Options:Close()
end

function KuronaBars:round(num, idp)
local outnum = num - num % 10

return outnum
end


--Path
function KuronaBars:RedrawPaths()
	if not self.player then return end
	
	local tAbilities = AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Path)
	if not tAbilities then
		return	
	end
	
	local wndList = self.wndPathPopoutFrame
	wndList:DestroyChildren()
	
	local nCount = 0
	local nListHeight = 0
	if self.tSettings.nSelectedPathId == nil then
		self.tSettings.nSelectedPathId = ActionSetLib.GetCurrentActionSet()[10]
	end
	local height = self.tSettings.SavedButtons["PathButton"].Button:GetHeight()			
	local width = self.tSettings.SavedButtons["PathButton"].Button:GetWidth()			
	
	for _, tAbility in pairs(tAbilities) do
		if tAbility.bIsActive then
			local splCurr = tAbility.tTiers[tAbility.nCurrentTier].splObject
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "PathBtn", wndList, self)
			nCount = nCount + 1
			
			self.tSettings.nSelectedPathId = self.tSettings.nSelectedPathId and self.tSettings.nSelectedPathId or tAbility.nId
						
			local nLeft, nTop, nRight, nBottom = wndCurr:GetAnchorOffsets()
			nListHeight = nListHeight + wndCurr:GetHeight()
			wndCurr:FindChild("PathBtnIcon"):SetSprite(splCurr:GetIcon())
			
			wndCurr:SetData(tAbility.nId)
			
			if self.tSettings.nSelectedPathId == tAbility.nId then
				self.PathButton:FindChild("Icon"):SetSprite(splCurr:GetIcon())	
			end
			
			if Tooltip and Tooltip.GetSpellTooltipForm and splCurr then
				wndCurr:SetTooltipDoc(nil)
				Tooltip.GetSpellTooltipForm(self, wndCurr, splCurr)
			end
			wndCurr:SetAnchorOffsets(0,0,width,width)
		end
		
	end
	if self.tSettings.nSelectedPathId ~= ActionSetLib.GetCurrentActionSet()[10] then
			self:HelperSetPathAbility(self.tSettings.nSelectedPathId)
		end
	
	self.bHasPathAbilities = nCount > 1
	
	local nHeight = wndList:ArrangeChildrenVert(0)

	local nLeft, nTop, nRight, nBottom = self.wndPathPopoutFrame:GetAnchorOffsets()
	self.wndPathPopoutFrame:SetAnchorOffsets(nLeft, nBottom - nHeight, nRight, nBottom)
	self.PathButton:FindChild("ArrowUpButton"):Show(#wndList:GetChildren() > 1)
end

function KuronaBars:OnPathBtn(wndHandler, wndControl)
	self.wndPathPopoutFrame:Show(false,false)
	local result = self:HelperSetPathAbility(wndControl:GetData())
	self.tSettings.nSelectedPathId = result and wndControl:GetData() or nil
	self:RedrawPaths()
end

function KuronaBars:HelperSetPathAbility(nAbilityId)
	local tActionSet = ActionSetLib.GetCurrentActionSet()
	if not tActionSet or not nAbilityId then
		return false
	end
	
	tActionSet[10] = nAbilityId
	local tResult = ActionSetLib.RequestActionSetChanges(tActionSet)

	if tResult.eResult ~= ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
		return false
	end
	
	Event_FireGenericEvent("PathAbilityUpdated", nAbilityId)
	self.tSettings.nSelectedPathId = nAbilityId
	
	return true
end


function KuronaBars:OnRecallBtn( wndHandler, wndControl, eMouseButton )
	self.wndRecallPopoutFrame:Show(false,false)
	local nRecallCommand = wndControl:GetData()
	
	GameLib.SetDefaultRecallCommand(nRecallCommand)
	self.tSettings.SavedButtons["RecallButton"].Content:SetContentId(nRecallCommand)
	--self.tSettings.SavedButtons["RecallButton"].Button:Show(false)
	self.tSettings.nSelectedRecall = nRecallCommand
	self.RecallButton:FindChild("Icon"):SetSprite(wndHandler:GetContent().spell:GetIcon())
	self:RedrawRecall()
end


function KuronaBars:RedrawRecall()
	self.wndRecallPopoutFrame:DestroyChildren() -- todo: selectively destroy the list
	--local nWndLeft, nWndTop, nWndRight, nWndBottom = self.wndMenu:GetAnchorOffsets()
	local nEntryHeight = 0
	local bHasBinds = false
	local bHasWarplot = false
	local guildCurr = nil
	
	local height = self.tSettings.SavedButtons["RecallButton"].Button:GetHeight()			
	local width = self.tSettings.SavedButtons["RecallButton"].Button:GetWidth()			

	-- todo: condense this 
	if GameLib.HasBindPoint() == true then
		--load recall
		local wndBind = Apollo.LoadForm(self.xmlDoc, "RecallBtn", self.wndRecallPopoutFrame, self)
		wndBind:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.BindPoint)
		wndBind:FindChild("RecallActionBtn"):SetData(GameLib.CodeEnumRecallCommand.BindPoint)
		wndBind:SetAnchorOffsets(0,0,width,width)
		bHasBinds = true
		local nLeft, nTop, nRight, nBottom = wndBind:GetAnchorOffsets()
		nEntryHeight = nEntryHeight + (nBottom - nTop)

	end
	if HousingLib.IsResidenceOwner() == true then
		-- load house
		local wndHouse = Apollo.LoadForm(self.xmlDoc, "RecallBtn", self.wndRecallPopoutFrame, self)
		wndHouse:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.House)
		wndHouse:FindChild("RecallActionBtn"):SetData(GameLib.CodeEnumRecallCommand.House)
		wndHouse:SetAnchorOffsets(0,0,width,width)
		bHasBinds = true
		local nLeft, nTop, nRight, nBottom = wndHouse:GetAnchorOffsets()
		nEntryHeight = nEntryHeight + (nBottom - nTop)		
	end

	-- Determine if this player is in a WarParty
	for key, guildCurr in pairs(GuildLib.GetGuilds()) do
		if guildCurr:GetType() == GuildLib.GuildType_WarParty then
			bHasWarplot = true
			break
		end
	end
	
	if bHasWarplot == true then
		-- load warplot
		local wndWarplot = Apollo.LoadForm(self.xmlDoc, "RecallBtn", self.wndRecallPopoutFrame, self)
		wndWarplot:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Warplot)
		wndWarplot:FindChild("RecallActionBtn"):SetData(GameLib.CodeEnumRecallCommand.Warplot)
		wndWarplot:SetAnchorOffsets(0,0,width,width)
		bHasBinds = true
		local nLeft, nTop, nRight, nBottom = wndWarplot:GetAnchorOffsets()
		nEntryHeight = nEntryHeight + (nBottom - nTop)	
	end
	
	local bIllium = false
	local bThayd = false
	
	for idx, tSpell in pairs(AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Misc) or {}) do
		if tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportIlliumSpell():GetBaseSpellId() then
			bIllium = true
		end
		if tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportThaydSpell():GetBaseSpellId() then
			bThayd = true
		end
	end
	
	if bIllium then
		-- load capital
		local wndWarplot = Apollo.LoadForm(self.xmlDoc, "RecallBtn", self.wndRecallPopoutFrame, self)
		wndWarplot:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Illium)
		wndWarplot:FindChild("RecallActionBtn"):SetData(GameLib.CodeEnumRecallCommand.Illium)
		wndWarplot:SetAnchorOffsets(0,0,width,width)
		
		bHasBinds = true
		local nLeft, nTop, nRight, nBottom = wndWarplot:GetAnchorOffsets()
		nEntryHeight = nEntryHeight + (nBottom - nTop)
	end
	
	if bThayd then
		-- load capital
		local wndWarplot = Apollo.LoadForm(self.xmlDoc, "RecallBtn", self.wndRecallPopoutFrame, self)
		wndWarplot:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Thayd)
		wndWarplot:FindChild("RecallActionBtn"):SetData(GameLib.CodeEnumRecallCommand.Thayd)		
		wndWarplot:SetAnchorOffsets(0,0,width,width)
		
		bHasBinds = true
		local nLeft, nTop, nRight, nBottom = wndWarplot:GetAnchorOffsets()
		nEntryHeight = nEntryHeight + (nBottom - nTop)
	end
	
	if bHasBinds == true then
		self.wndRecallPopoutFrame:SetText("")
		
		local nHeight = self.wndRecallPopoutFrame:ArrangeChildrenVert(0)
		local nLeft, nTop, nRight, nBottom = self.wndRecallPopoutFrame:GetAnchorOffsets()
		self.wndRecallPopoutFrame:SetAnchorOffsets(nLeft, nBottom - nHeight, nRight, nBottom)
	end
end

function KuronaBars:RefreshDefaultCommand()
if self.RecallButton == nil then return end
	if GameLib.GetDefaultRecallCommand() == nil then
		self:ResetDefaultCommand()
	elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.BindPoint then
		if GameLib.HasBindPoint() == false then 	
			self:ResetDefaultCommand()
		end
	elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.House then
		if HousingLib.IsResidenceOwner() == false then 	
			self:ResetDefaultCommand()
		end
	elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Warplot then
		local bNeedsReset = true
		-- Determine if this player is in a WarParty
		for key, guildCurr in pairs(GuildLib.GetGuilds()) do
			if guildCurr:GetType() == GuildLib.GuildType_WarParty then
				bNeedsReset = false
				break
			end
		end
		if bNeedsReset then
			self:ResetDefaultCommand()
		end
	elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Illium then
		local bNeedsReset = false
		for idx, tSpell in pairs(AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Misc) or {}) do
			if not tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportIlliumSpell():GetBaseSpellId() then
				bNeedsReset = true
			end
		end
		if bNeedsReset then
			self:ResetDefaultCommand()
		end
	elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Thayd then
		local bNeedsReset = false
		for idx, tSpell in pairs(AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Misc) or {}) do
			if not tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportThaydSpell():GetBaseSpellId() then
				bNeedsReset = true
			end
		end
		if bNeedsReset then
			self:ResetDefaultCommand()
		end
	end
	
	local bShowRecallBtn = false
	if GameLib.GetDefaultRecallCommand() ~= nil then
		if GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.BindPoint then
			if GameLib.HasBindPoint() then
				bShowRecallBtn = true
				self.RecallButton:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.BindPoint)
			else
				bShowRecallBtn = false
			end
		elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.House then
			bShowRecallBtn = true
			self.RecallButton:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.House)
		elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Warplot then
			bShowRecallBtn = true
			self.RecallButton:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Warplot)
		elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Illium then
			bShowRecallBtn = true
			self.RecallButton:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Illium)
		elseif GameLib.GetDefaultRecallCommand() == GameLib.CodeEnumRecallCommand.Thayd then
			bShowRecallBtn = true
			self.RecallButton:FindChild("RecallActionBtn"):SetContentId(GameLib.CodeEnumRecallCommand.Thayd)
		end
	else
		bShowRecallBtn = false
	end
	
	self.tSettings.SavedButtons["RecallButton"].Button:Show(bShowRecallBtn)
end

function KuronaBars:ResetDefaultCommand()
	local bHasWarplot = false
	-- Determine if this player is in a WarParty
	for key, guildCurr in pairs(GuildLib.GetGuilds()) do
		if guildCurr:GetType() == GuildLib.GuildType_WarParty then
			bHasWarplot = true
			break
		end
	end
	
	local bHasIllium = false
	local bHasThyad = false
	for idx, tSpell in pairs(AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Misc) or {}) do
		if tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportThaydSpell():GetBaseSpellId() then
			bHasThyad = true
		end
		if tSpell.bIsActive and tSpell.nId == GameLib.GetTeleportIlliumSpell():GetBaseSpellId() then
			bHasIllium = true
		end
	end

	if GameLib.HasBindPoint() then 	
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.BindPoint)
	elseif HousingLib.IsResidenceOwner() == true then
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.House)	
	elseif bHasWarplot then
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.Warplot)
	elseif bHasIllium then
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.Illium)
	elseif bHasThyad then
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.Thayd)
	else
		GameLib.SetDefaultRecallCommand(GameLib.CodeEnumRecallCommand.BindPoint)
	end
end


--Floating Shortcut
function KuronaBars:RedrawShortcuts()
	self:RedrawActionBarShortcut()
end

function KuronaBars:RedrawActionBarShortcut()

if self.tSettings.FloatBarPos then
	self.FloatingActionBar:SetAnchorOffsets(unpack(self.tSettings.FloatBarPos))
end

	local tShortcutCount = {}
	
	self.tActionBarSettings = {}
	--Floating Bar - Horizontal
	self.tActionBarsHorz = {}
	for idx = 4, ActionSetLib.CodeEnumShortcutSet.Count do
		local wndCurrBar = Apollo.LoadForm(self.xmlDoc, "ActionBarContainer", self.FloatingActionBar, self)
		wndCurrBar:Show(false,true)
		for iBar = 0, 7 do
			local wndBarItem = Apollo.LoadForm(self.xmlDoc, "ActionBarShortcutItem", wndCurrBar:FindChild("ActionBarContainer"), self)
			wndBarItem:FindChild("ActionBarShortcutBtn"):SetContentId(idx * 12 + iBar)
			if wndBarItem:FindChild("ActionBarShortcutBtn"):GetContent()["strIcon"] ~= "" then
				tShortcutCount[idx] = iBar + 1
				wndBarItem:Show(true,true)
			end
			wndCurrBar:FindChild("ActionBarContainer"):ArrangeChildrenHorz(0)
		end

--		wndCurrBar:FindChild("DockBtn"):SetCheck(not self.bDocked)
--		wndCurrBar:FindChild("OrientationBtn"):SetCheck(not self.bHorz)
		self.tActionBarsHorz[idx] = wndCurrBar
	end
	self.FloatingActionBar:Show(true)
end

function KuronaBars:ShowABWindow(nBar, bIsVisible, nShortcuts)
	if nBar == ActionSetLib.CodeEnumShortcutSet.VehicleBar and self.Bar1 and self.Bar1:IsValid() then
		self:ShowVehicleBar(nBar, bIsVisible, nShortcuts) -- show/hide vehicle bar if eWhichBar matches
		return
	end
	
    if self.tActionBarsHorz[nBar] == nil then
		return
	end

	self.tActionBarSettings[nBar] = {}
	self.tActionBarSettings[nBar].bIsVisible = bIsVisible
	self.tActionBarSettings[nBar].nShortcuts = nShortcuts

	local l,t,r,b = self.FloatingActionBar:GetAnchorOffsets()
	local halfwidth = (40 + (nShortcuts*48)) / 2
	self.FloatingActionBar:SetAnchorOffsets(-halfwidth,t,halfwidth,t+90)
	self.tActionBarsHorz[nBar]:Show(bIsVisible)
end

function KuronaBars:ShowVehicleBar(eWhichBar, bIsVisible, nNumShortcuts)
	if eWhichBar ~= ActionSetLib.CodeEnumShortcutSet.VehicleBar or not self.Bar1 or not self.Bar1:IsValid() then
		return
	end
	--self.Bar1:FindChild("ContentWindow"):Show(not bIsVisible)
	local wndVehicleBar = self.VehicleBarFrame
	wndVehicleBar:Show(bIsVisible)
	self.VehicleBarShown = bIsVisible

	for i =1,9 do
		if self.tSettings.SavedButtons["Button"..i] and self.tSettings.SavedButtons["Button"..i].Button then
			self.tSettings.SavedButtons["Button"..i].Button:Show(not bIsVisible)
			self.tSettings.SavedButtons["Button"..i].Button:FindChild("ActionBarBtn"):Enable(not bIsVisible)
		end
	end
			self.tSettings.SavedButtons["PotionButton"].Button:Show(not bIsVisible)
			self.tSettings.SavedButtons["StanceButton"].Button:Show(not bIsVisible)
			self.tSettings.SavedButtons["PathButton"].Button:Show(not bIsVisible)
		
	local unitPlayer = GameLib:GetPlayerUnit()
	if unitPlayer and not unitPlayer:IsInVehicle() then
		self.tCurrentVehicleInfo = nil
	end

	if bIsVisible then
		for idx = 1, 6 do -- TODO hardcoded formatting
			wndVehicleBar:FindChild("ActionBarShortcutContainer" .. idx):Show(false)
		end

		if nNumShortcuts then
			for idx = 1, nNumShortcuts do
				wndVehicleBar:FindChild("ActionBarShortcutContainer" .. idx):Show(true)
				wndVehicleBar:FindChild("ActionBarShortcutContainer" .. idx):FindChild("ActionBarShortcut." .. idx):Enable(true)
			end

			local nLeft, nTop ,nRight, nBottom = wndVehicleBar:GetAnchorOffsets() -- TODO SUPER HARDCODED FORMATTING
			wndVehicleBar:SetAnchorOffsets(nLeft, nTop, nLeft + (58 * nNumShortcuts) + 66, nBottom)
		end

		wndVehicleBar:ArrangeChildrenHorz(1)
		
		self.tCurrentVehicleInfo =
		{
			nBar = nWhichBar,
			nNumShortcuts = nNumShortcuts,
		}
	end
end


function KuronaBars:ManualAdjust( wndHandler, wndControl, eMouseButton )
	if self.LastButton == nil then return end
	
		local calcWidth = math.floor((self.LastButton:GetScale() * 50) + 0.5 )
		self.EditModeForm:FindChild("WidthBox"):SetText(calcWidth)
		self.EditModeForm:FindChild("ButtonWidthSliderBar"):SetValue(calcWidth)

	
	local name = wndHandler:GetName()
	local l,t,r,b = self.LastButton:GetAnchorOffsets()
	local width = self.LastButton:GetWidth()
	local height = self.LastButton:GetHeight()
	if width >= 50 or height >= 50 then
		local scale = self.LastButton:GetScale()
--		r = math.ceil(r * scale)
--		b = math.ceil(b * scale)
	end
	self.ControlType = self.Controls:GetRadioSel("PosSize")
	
	if self.ControlType == 1 then
	if name == "UpButton" then
		self.LastButton:SetAnchorOffsets(l,t-1,r,b-1)
	elseif name == "DownButton" then
		self.LastButton:SetAnchorOffsets(l,t+1,r,b+1)
	elseif name == "RightButton" then
		self.LastButton:SetAnchorOffsets(l+1,t,r+1,b)
	elseif name == "LeftButton" then
		self.LastButton:SetAnchorOffsets(l-1,t,r-1,b)
		
	end
	
	end
	local l,t,r,b = self.LastButton:GetAnchorOffsets()
		--r = l + calcWidth
		--b = t + calcWidth
	self.EditModeForm:FindChild("LeftEditBox"):SetText(l)
	self.EditModeForm:FindChild("UpEditBox"):SetText(t)
	self.EditModeForm:FindChild("RightEditBox"):SetText(r)
	self.EditModeForm:FindChild("DownEditBox"):SetText(b)
	
end

function KuronaBars:OnButtonWidthSlider( wndHandler, wndControl, fNewValue, fOldValue )
	self.LastButton:SetScale(fNewValue / 50)
	local calcWidth = math.floor((self.LastButton:GetScale() * 50) + 0.5 )
	self.EditModeForm:FindChild("WidthBox"):SetText(calcWidth)
	--self.EditModeForm:FindChild("ButtonWidthSliderBar"):SetValue(calcWidth)
	self:ManualAdjust(self.LastButton,self.LastButton)
end

function KuronaBars:OnEnteredCombat(unit, bInCombat)
	if unit == GameLib:GetPlayerUnit() and bInCombat then
	end
end


function KuronaBars:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "KuronaBars", {"KuronaBarsClick", "", "KBarSprite:KBInterfaceIcon"})
end


function KuronaBars:OnKuronaBarsMenuClick()
	if not self.Options:IsShown() then
		self:OnKuronaBarsOn()
	elseif self.Options:IsShown() then
		self:OptionsClose()
	end
end
