-----------------------------------------------------------------------------------------------
-- Client Lua Script for KuronaMounts
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- KuronaMounts Module Definition
-----------------------------------------------------------------------------------------------
local KuronaMounts = {} 
local Version = 1
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function KuronaMounts:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function KuronaMounts:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- KuronaMounts OnLoad
-----------------------------------------------------------------------------------------------
function KuronaMounts:OnLoad()
    -- load our form file
	--self.nSelectedMount = nil

	self.xmlDoc = XmlDoc.CreateFromFile("KuronaMounts.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("KPSprite.xml","KPSprite")
	
end

-----------------------------------------------------------------------------------------------
-- KuronaMounts OnDocLoaded
-----------------------------------------------------------------------------------------------
function KuronaMounts:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "KuronaMountsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.tKnownPets = {}
		self.tUnknownPets = {}
	
		self.bShowUnknown = false

		
		self.MountFlyout = Apollo.LoadForm(self.xmlDoc, "MountFlyout", "FixedHudStratum", self)
		self.MountFlyoutFrame = Apollo.LoadForm(self.xmlDoc, "MountPopupFrame", nil, self)
		if self.MountFlyoutFrame ~= nil then
			self.MountFlyout:FindChild("MountPopoutBtn"):SetCheck(false)
			self.MountFlyout:FindChild("MountPopoutBtn"):AttachWindow(self.MountFlyoutFrame)
		end
		
		--defaults
		
		if self.tPos then
			self.MountFlyout:SetAnchorOffsets(unpack(self.tPos))
		end
		self:MoveFlyout()	
		
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
		Apollo.RegisterTimerHandler("RedrawTimer", "RedrawMounts", self)
		Apollo.RegisterEventHandler("GenericEvent_CollectablesReady", "OnReady", self)
		Apollo.RegisterEventHandler("AbilityBookChange", "RedrawMounts", self)
		Apollo.RegisterSlashCommand("kmounts", "OnConsole", self)
		
		self.SummonTries = 0
		
		self:FillOptions()
		
	    self.wndMain:Show(false, true)
	end
	
	if GameLib.GetPlayerUnit() ~= nil then
		self:OnCharacterCreated()
	end

end

-----------------------------------------------------------------------------------------------
-- KuronaMounts Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here


-----------------------------------------------------------------------------------------------
-- KuronaMountsForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function KuronaMounts:OnOK()
	self.wndMain:Close() -- hide the window
	self.MountFlyout:SetStyle("RequireMetaKeyToMove",false)
	self.MountFlyout:FindChild("MountPopoutBtn"):Enable(true)
	self.MountFlyout:SetStyle("Picture",false)	
	self.MountFlyout:SetStyle("IgnoreMouse",true)	
end

-- when the Cancel button is clicked
function KuronaMounts:OnCancel()
	self:OnOK() -- hide the window
end

function KuronaMounts:PopulateMountList()

end


---------------------------------------------------------------------------------------------------
-- PetBtn Functions
---------------------------------------------------------------------------------------------------

function KuronaMounts:OnGenerateTooltip(wndControl, wndHandler, eType, arg1, arg2)
	local xml = nil
	if eType == Tooltip.TooltipGenerateType_ItemInstance then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_ItemData then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_GameCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Macro then
		xml = XmlDoc.new()
		xml:AddLine(arg1)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Spell then
		if Tooltip ~= nil and Tooltip.GetSpellTooltipForm ~= nil then
			Tooltip.GetSpellTooltipForm(self, wndControl, arg1)
		end
	elseif eType == Tooltip.TooltipGenerateType_PetCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	end
end


-----------------------------------------------------------------------------------------------
-- KuronaMounts Instance
-----------------------------------------------------------------------------------------------
local KuronaMountsInst = KuronaMounts:new()
KuronaMountsInst:Init()




function KuronaMounts:RedrawMounts()
	local wndMountPopout = self.MountFlyoutFrame:FindChild("MountPopoutList")
	wndMountPopout:DestroyChildren()

	local tMountList = CollectiblesLib.GetMountList()
	local splSelected = nil

	for idx, tMountData  in pairs(tMountList) do
		if tMountData.bIsKnown then
			local splMount = tMountData.splObject
			
			if not splSelected then
				splSelected = splMount
			end

			if splMount:GetId() == self.nSelectedMount then
				splSelected = splMount
			end

			local wndCurr = Apollo.LoadForm(self.xmlDoc, "MountBtn", wndMountPopout, self)
			wndCurr:FindChild("MountBtnIcon"):SetSprite(splMount:GetIcon())
			wndCurr:SetData(splMount)

			if Tooltip and Tooltip.GetSpellTooltipForm then
				wndCurr:SetTooltipDoc(nil)
				Tooltip.GetSpellTooltipForm(self, wndCurr, splMount, {})
			end
		end
	end

	if splSelected then
		GameLib.SetShortcutMount(splSelected:GetId())
	end

	local nCount = #wndMountPopout:GetChildren()
	if nCount > 0 then
		local nMax = 7
		local nMaxHeight = (wndMountPopout:ArrangeChildrenVert(0) / nCount) * nMax
		local nHeight = wndMountPopout:ArrangeChildrenVert(0)

		nHeight = nHeight <= nMaxHeight and nHeight or nMaxHeight

		local nLeft, nTop, nRight, nBottom = self.MountFlyoutFrame:GetAnchorOffsets()

		self.MountFlyoutFrame:SetAnchorOffsets(nLeft, nBottom - nHeight - 74, nRight, nBottom)
--		self:RedrawBarVisibility()
	else
		--self.MountFlyout:Show()
	end
end

function KuronaMounts:OnMountBtn(wndHandler, wndControl)
	self.nSelectedMount = wndControl:GetData():GetId()

	self.MountFlyoutFrame:Show(false)
	self:RedrawSelectedMounts()
end


function KuronaMounts:OnReady()
	self:RedrawMounts()
end

function KuronaMounts:RedrawSelectedMounts()
	GameLib.SetShortcutMount(self.nSelectedMount)
end



function KuronaMounts:RedrawSelectedPet()
	self.MountFlyout:FindChild("Icon"):SetSprite(self.nSelectedPet.splObject and self.nSelectedPet.splObject:GetIcon())
end

function KuronaMounts:OnCharacterCreated()
	Apollo.CreateTimer("RedrawTimer", 2, false)
end

function KuronaMounts:ArrangeList()
	local wndPetList = self.MountFlyoutFrame:FindChild("PetPopoutList")
	
	wndPetList:ArrangeChildrenVert()
	wndPetList:SetVScrollPos(0)
	--self.MountFlyoutFrame:RecalculateContentExtents()
end

function KuronaMounts:OnSummonPet( wndHandler, wndControl, eMouseButton )
if self.nSelectedPet == nil then return end
	local arPets = GameLib.GetPlayerPets()
	local bPetSummoned = false
	if arPets then
		for idx, unitPet in pairs (arPets) do
			if unitPet:GetName() == self.nSelectedPet.strName then
				bPetSummoned = true
				break
			end
		end
	end

	if not bPetSummoned or eMouseButton== 1 then
		GameLib.SummonVanityPet(self.nSelectedPet.nId)
		if self.bCanTalk then
			Apollo.CreateTimer("TalkTimer", 3, false)
		end
	end
end


function KuronaMounts:OnSave(eLevel,backup)	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tsave = {}
	tsave.nSelectedMount = self.nSelectedMount
	tsave.tPos = {self.MountFlyout:GetAnchorOffsets()}
	return tsave
end

function KuronaMounts:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	self.nSelectedMount = saveData.nSelectedMount
	self.tPos = saveData.tPos
	end


function KuronaMounts:FillOptions()
end

function KuronaMounts:OnConsole()
	self.wndMain:Invoke()
	self.MountFlyout:SetStyle("RequireMetaKeyToMove",false)
	self.MountFlyout:SetStyle("Picture",true)
	self.MountFlyout:SetStyle("IgnoreMouse",false)
	self.MountFlyoutFrame:Show(false)
	self.MountFlyout:FindChild("MountPopoutBtn"):Enable(false)
end

function KuronaMounts:OnSettingsChange( wndHandler, wndControl, eMouseButton )
	self[wndHandler:GetName()] = wndHandler:IsChecked()
end

function KuronaMounts:OnMouseOver( wndHandler, wndControl, x, y )
--	wndHandler:FindChild("Icon"):SetBGColor("ffffff00")
--	Print("over")
end

function KuronaMounts:OnMouseExit( wndHandler, wndControl, x, y )
--	wndHandler:FindChild("Icon"):SetBGColor("ffffffff")
--	Print("exit")
end


function KuronaMounts:ResetPos( wndHandler, wndControl, eMouseButton )
	self.MountFlyout:SetAnchorOffsets(4,-113,96,-32)
	self:MoveFlyout()
end

function KuronaMounts:MoveFlyout()
	self.tPos = {self.MountFlyout:GetAnchorOffsets()}
	local l,t,r,b = self.tPos[1],self.tPos[2],self.tPos[3],self.tPos[4]
	self.MountFlyoutFrame:SetAnchorOffsets(l-16,t-448,r+26,b-76)
end
